//============================================================================
//
//    Parallel checker
//
//    Kit.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.text.*;

import static parallel.Constant.*;

public class Kit
{
    public static String HOME;
    public static JFrame mainFrame;
    public static JTextPane textPane;

    //////////////////////////////////////////////////////////////////////////

    public static void print(String ln)
    {
        try
        {
            SimpleAttributeSet attr = new SimpleAttributeSet();

            Document doc = textPane.getDocument();
            int len = doc.getLength();
            doc.insertString(len, ln + "\n", attr);
            len += ln.length() + 1;
            textPane.setCaretPosition(len);
        }
        catch(Exception ex) {}
    }

    //////////////////////////////////////////////////////////////////////////

    static ArrayList<Object[]> errors = new ArrayList<Object[]>();

    public static void clearErr()
    {
        errors.clear();
    }

    public static void addErr(String      msg,
                              OnPointPair pt1,
                              OnPointPair pt2)
    {
        errors.add(new Object[] { msg, pt1, pt2 });
    }

    public static void putErr()
    {
        Collections.sort(errors,
                        new Comparator<Object[]>()
                        {
                            public int compare(Object [] p1, Object [] p2)
                            {
                                OnPointPair pt1 = (OnPointPair)p1[1];
                                OnPointPair pt2 = (OnPointPair)p2[1];
                                return pt1.note1.bar.no - pt2.note1.bar.no;
                            }
                        }
        );

        for(int i = 0; i < errors.size(); ++i)
        {
            Object[] p = errors.get(i);
            String      msg = (String)p[0];
            OnPointPair pt1 = (OnPointPair)p[1];
            OnPointPair pt2 = (OnPointPair)p[2];
            printErr(msg, pt1, pt2);
        }
    }

    public static void printErr(String      msg,
                                OnPointPair pt1,
                                OnPointPair pt2)
    {
        Kit.print("---------------------------------");
        Kit.print(String.format("%s  トラック %d & %d",
                                    msg, pt1.note1.track+1,  pt1.note2.track+1));

        Kit.print(pt1.toString());
        Kit.print(pt2.toString());

        if (traceOn)
        {
            System.err.printf("---------------------------------\r\n");
            System.err.printf("%s  トラック %d & %d\r\n",
                                        msg, pt1.note1.track+1,  pt1.note2.track+1);

            System.err.printf("bar.ttl = %d,  pos = %d, unit = %s\r\n",
                                             pt1.note1.bar.totalTick,
                                             pt1.pos,
                                             24/pt1.note1.bar.divisions);
            System.err.printf("%s\r\n", pt1.toString());
            System.err.printf("%s\r\n", pt2.toString());
        }
    }

    //////////////////////////////////////////////////////////////////////////

    static Hashtable<String, ImageIcon> iconMap = new Hashtable<String, ImageIcon>();

    public static ImageIcon getIconImage(String k)
    {
        String nk = "/resource/" + k;
        if (iconMap.containsKey(nk))
            return iconMap.get(nk);

        URL url = Kit.class.getResource(nk);
        if (url ==  null)
            return null;
        ImageIcon icon = new ImageIcon(url);
        iconMap.put(nk, icon);
        return icon;
    }

    //////////////////////////////////////////////////////////////////////////

    public static void setWaitCursor(boolean flag)
    {
        Cursor setting = Cursor.getPredefinedCursor((flag)? Cursor.WAIT_CURSOR
                                                          : Cursor.DEFAULT_CURSOR);
        try
        {
            mainFrame.setCursor(setting);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    //////////////////////////////////////////////////////////////////////////

    static final Object[] optionsYesNo = { "了解",
                                           "取消" };

    public static boolean yesNoDialog(String message, String title)
    {
        int result = JOptionPane.showOptionDialog(mainFrame, message, title,
                                                  JOptionPane.DEFAULT_OPTION,
                                                  JOptionPane.ERROR_MESSAGE,
                                                  null,
                                                  optionsYesNo,
                                                  optionsYesNo[0]);
        return result == 0;
    }

    //////////////////////////////////////////////////////////////////////////

    public static void setFont(Container c, Font f)
    {
        if (c instanceof JMenuBar)
            return;

        if (c instanceof AbstractButton
         || c instanceof JLabel
         || c instanceof JTextPane)
        {
            c.setFont(f);
        }

        Component [] cps = c.getComponents();
        for (Component cp : cps)
        {
            if (cp instanceof Container)
                setFont((Container)cp, f);
        }
    }
}
