//============================================================================
//
//  Parallel checker
//
//  MainFrame.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

public class MainFrame extends JFrame
{
    // Menue
    JMenuItem openItem;
    JMenuItem saveItem;
    JMenuItem exitItem;
    JMenuItem optionItem;
    // ToolBar
    JButton   openBtn;
    JButton   saveBtn;
    JButton   runBtn;
    JButton   clearBtn;
    // TextPane
    JTextPane text = null;
    // StatusBar
    JTextField statusField;

    File loadHolder = null;

    Checker checker = new Checker();

    public MainFrame()
    {
        super();
        createInitFrame();

        ArrayList imgs = new ArrayList();
        imgs.add(new ImageIcon(Main.class.getResource("/resource/Icon16x16.gif")).getImage());
        imgs.add(new ImageIcon(Main.class.getResource("/resource/Icon32x32.gif")).getImage());
        this.setIconImages(imgs);

        Kit.textPane = text;
        Options.load();
    }

    private void createInitFrame()
    {
        setTitle("Parallel Checker");

        JPanel center = new JPanel();
        center.setLayout(new BorderLayout());
        text = new JTextPane();
        text.setEditable(false);
        JScrollPane scroll1 = new JScrollPane(text);
        center.add(scroll1, BorderLayout.CENTER);

        statusField = new JTextField();
        statusField.setEditable(false);

        JMenuBar menubar = createMenuBar();
        setJMenuBar(menubar);

        JToolBar toolbar = createToolBar();
        JPanel toolbarBanel = new JPanel();
        toolbarBanel.setLayout(new BorderLayout());
        toolbarBanel.add(toolbar, BorderLayout.CENTER);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(toolbarBanel,    BorderLayout.NORTH);
        getContentPane().add(center,          BorderLayout.CENTER);
        getContentPane().add(statusField,     BorderLayout.SOUTH);
        getContentPane().setPreferredSize(new Dimension(650, 600));

        pack();

        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(size.width /2 - getSize().width /2,
                    size.height/2 - getSize().height/2);

        Font font = Font.decode("DialogInput-PLAIN-15");
        Kit.setFont(text, font);

        setListener();
    }

    private void setListener()
    {
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE );

        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e){ close(); }
        });

        ActionListener openAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                openFile();
            }
        };
        openItem.addActionListener(openAct);
        openBtn.addActionListener(openAct);

        ActionListener saveAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                saveCurrent();
            }
        };
        saveItem.addActionListener(saveAct);
        saveBtn.addActionListener(saveAct);

        exitItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                close();
            }
        });

        optionItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                openOptionDialog();
            }
        });

        runBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                checkStart();
            }
        });

        clearBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                text.setText("");
            }
        });

        DropTargetListener dtl = new DropTargetAdapter()
        {
           public void dragOver(DropTargetDragEvent dtde)
           {
                if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
                {
                    dtde.acceptDrag(DnDConstants.ACTION_COPY);
                    return;
                }
                dtde.rejectDrag();
            }

            public void drop(DropTargetDropEvent dtde)
            {
                try
                {
                    if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
                    {
                        dtde.acceptDrop(DnDConstants.ACTION_COPY);
                        Transferable transferable = dtde.getTransferable();
                        java.util.List list = (java.util.List)(transferable.getTransferData(DataFlavor.javaFileListFlavor));
                        for (int i = 0;i < list.size(); ++i)
                             fileDrop((File)list.get(i));

                        dtde.dropComplete(true);
                        return;
                    }
                }
                catch(UnsupportedFlavorException ufe)
                {
                    ufe.printStackTrace();
                }
                catch(IOException ioe)
                {
                    ioe.printStackTrace();
                }
                dtde.rejectDrop();
            }
        };
        new DropTarget(text, DnDConstants.ACTION_COPY, dtl, true);
    }

    public void close()
    {
        setVisible(false);
        this.dispose();
        System.exit(0);
    }

    protected JMenuBar createMenuBar()
    {
        JMenuBar menuBar  = new JMenuBar();
        JMenu menuFile    = new JMenu("ファイル(F)");
        JMenu menuSetting = new JMenu("設定(S)");
        openItem = new JMenuItem("開く(O)",      Kit.getIconImage("OpenFile.GIF"));
        saveItem = new JMenuItem("上書き保存(S)", Kit.getIconImage("SaveFile.GIF"));
        exitItem = new JMenuItem("終了(X)");

        menuFile.setMnemonic(KeyEvent.VK_F);
        menuSetting.setMnemonic(KeyEvent.VK_E);
        openItem.setMnemonic(KeyEvent.VK_O);
        openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
        saveItem.setMnemonic(KeyEvent.VK_S);
        saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        exitItem.setMnemonic(KeyEvent.VK_X);

        menuFile.add(openItem);
        menuFile.add(new JSeparator());
        menuFile.add(saveItem);
        menuFile.add(new JSeparator());
        menuFile.add(exitItem);

        optionItem = new JMenuItem("条件", Kit.getIconImage("Option.GIF"));
        menuSetting.add(optionItem);

        menuBar.add(menuFile);
        menuBar.add(menuSetting);
        return menuBar;
    }

    private JToolBar createToolBar()
    {
        JToolBar toolBar = new JToolBar("ツール");
        openBtn = new JButton (Kit.getIconImage("OpenFile.GIF"));
        openBtn.setToolTipText("開く");
        openBtn.setOpaque(false);
        saveBtn = new JButton (Kit.getIconImage("SaveFile.GIF"));
        saveBtn.setToolTipText("結果を保存");
        saveBtn.setOpaque(false);

        runBtn = new JButton (Kit.getIconImage("Run.GIF"));
        runBtn.setToolTipText("実行");
        runBtn.setOpaque(false);

        clearBtn = new JButton (Kit.getIconImage("Clear.GIF"));
        clearBtn.setToolTipText("結果をクリア");
        clearBtn.setOpaque(false);

        toolBar.add(openBtn);
        toolBar.add(saveBtn);
        toolBar.add(runBtn);
        toolBar.add(Box.createHorizontalGlue());

        toolBar.add(clearBtn);
        return toolBar;
    }

    public void resetStatusText()
    {
        setStatusText("");
    }

    public void setStatusText(final String s)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                statusField.setText(s);
            }
        });
    }

    void openFile()
    {
        JFileChooser fileChooser = new JFileChooser(loadHolder);
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setMultiSelectionEnabled(false);
        FileNameExtensionFilter filter = new FileNameExtensionFilter("MusicXMLファイル(*.xml)", "xml");
        fileChooser.addChoosableFileFilter(filter);
        fileChooser.setAcceptAllFileFilterUsed(false);

        if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
            return;

        File src = fileChooser.getSelectedFile();
        if (src != null)
            openFile(src);
    }

    private void fileDrop(File f)
    {
        if (f.isDirectory())
        {
            for(File ff : f.listFiles())
            {
                fileDrop(ff);
                break;
            }
        }
        else
        {
            openFile(f);
        }
    }

    public void openFile(File f)
    {
        loadHolder = f.getParentFile();
        checker.loadXML(f);
    }

    void saveCurrent()
    {
        JFileChooser fileChooser = new JFileChooser(loadHolder);
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        FileNameExtensionFilter filter = new FileNameExtensionFilter("Textファイル(*.txt)", "txt");
        fileChooser.addChoosableFileFilter(filter);
        fileChooser.setAcceptAllFileFilterUsed(false);

        if (fileChooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
            return;

        File dst = fileChooser.getSelectedFile();
        if (dst == null)
            return;

        if (dst.exists())
        {
            if (!Kit.yesNoDialog("ファイルが存在します. 上書きしますか?" + " ",
                                 "確認"))
                return;
        }

        try (BufferedWriter stream
             = new BufferedWriter(
                new OutputStreamWriter(
                 new FileOutputStream(dst), "UTF-8")) )
        {
            String s = text.getText();
            stream.write(s);
            stream.write("\n");
        }
        catch(Exception e)
        {
            Kit.print(e.getMessage());
        }
    }

    void checkStart()
    {
        checker.run();
    }

    void openOptionDialog()
    {
        OptionDialog dlg = new OptionDialog();
        dlg.setVisible(true);
    }
}
