//============================================================================
//
//    Parallel checker
//
//  OptionDialog.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class OptionDialog extends JDialog
{
    private JCheckBox parallel8Chk;
    private JCheckBox parallel5Chk;
    private JCheckBox useCancel8Chk;
    private JCheckBox useCancel5Chk;
    private JComboBox checkBeatsCombo;
    String[] beatData = {"2", "3", "4" };

    private JCheckBox reach8Chk;
    private JCheckBox reach5Chk;

    private JButton   okBtn;
    private JButton   cancelBtn;

    public OptionDialog()
    {
        super(Kit.mainFrame, "チェック条件の設定");

        ArrayList imgs = new ArrayList();
        imgs.add(new ImageIcon(Main.class.getResource("/resource/Icon16x16.gif")).getImage());
        imgs.add(new ImageIcon(Main.class.getResource("/resource/Icon32x32.gif")).getImage());
        this.setIconImages(imgs);

        setLayout(new BorderLayout());

        getContentPane().add(createPanel(), BorderLayout.CENTER);
        getContentPane().add(createBtns(), BorderLayout.SOUTH);

        Font font = Font.decode("DialogInput-PLAIN-14");
        Kit.setFont(this, font);

        setListener();

        setSize(400, 422);
        setResizable(false);

        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(size.width /2 - getSize().width /2,
                    size.height/2 - getSize().height/2);

        setFieldsInit();
    }

    protected JPanel createPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(null);
        int x = 10;
        int y = 10;

        parallel8Chk = new JCheckBox("連続8度をチェックする.");
        parallel8Chk.setBounds(x,y, 300, 24);
        panel.add(parallel8Chk);
        y += 36;

        parallel5Chk = new JCheckBox("連続5度をチェックする.");
        parallel5Chk.setBounds(x,y, 300, 24);
        panel.add(parallel5Chk);
        y += 36;

        useCancel8Chk = new JCheckBox("途中で解消音程(3,4,5,6度)があれば連続8度としない.");
        useCancel8Chk.setBounds(x,y, 500, 24);
        panel.add(useCancel8Chk);
        y += 36;

        useCancel5Chk = new JCheckBox("途中で解消音程(3,4,6度)があれば連続5度としない.");
        useCancel5Chk.setBounds(x,y, 500, 24);
        panel.add(useCancel5Chk);
        y += 36;

        JLabel l2 = new JLabel("チェック間隔(基本拍の倍数).");
        l2.setBounds(x, y , 200, 22);
        panel.add(l2);
        checkBeatsCombo = new JComboBox(new DefaultComboBoxModel(beatData));
        checkBeatsCombo.setBounds(x+210, y , 100, 22);
        panel.add(checkBeatsCombo);
        y += 36;

        reach8Chk = new JCheckBox("並達8度をチェックする.");
        reach8Chk.setBounds(x,y, 300, 24);
        panel.add(reach8Chk);
        y += 36;

        reach5Chk = new JCheckBox("並達5度をチェックする.");
        reach5Chk.setBounds(x,y, 300, 24);
        panel.add(reach5Chk);
        y += 36;

        return panel;
    }

    private JComponent createBtns()
    {
        JPanel p = new JPanel();
        okBtn = new JButton("了解");
        cancelBtn = new JButton("取消");

        p.add(Box.createHorizontalGlue());
        p.add(okBtn);
        p.add(cancelBtn);
        p.add(Box.createHorizontalGlue());
        return p;
    }

    protected void setListener()
    {
        okBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               settingOK();
            }
        });

        cancelBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               setVisible(false);
            }
        });
    }

    private void setFieldsInit()
    {
        parallel8Chk.setSelected(Options.parallel8);
        parallel5Chk.setSelected(Options.parallel5);
        useCancel8Chk.setSelected(Options.useCancel8);
        useCancel5Chk.setSelected(Options.useCancel5);
        checkBeatsCombo.setSelectedIndex(Options.checkBeats-2);
        reach8Chk.setSelected(Options.reach8);
        reach5Chk.setSelected(Options.reach5);
    }

    void settingOK()
    {
        Options.parallel8  = parallel8Chk.isSelected();
        Options.parallel5  = parallel5Chk.isSelected();
        Options.useCancel8 = useCancel8Chk.isSelected();
        Options.useCancel5 = useCancel5Chk.isSelected();
        Options.checkBeats = checkBeatsCombo.getSelectedIndex()+2;
        Options.reach8 = reach8Chk.isSelected();
        Options.reach5 = reach5Chk.isSelected();

        Options.save();
        setVisible(false);
    }
}
