//============================================================================
//
//    Parallel checker
//
//  Options.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.io.*;
import java.util.*;

public class Options
{
    private static final String FILE_NAME = "/options.properties";
    private static final String PARALLELE8_TAG = "parallel8";
    private static final String PARALLELE5_TAG = "parallel5";
    private static final String REACH8_TAG     = "reach8";
    private static final String REACH5_TAG     = "reach5";
    private static final String CHECKBEATS_TAG = "checkBeats";
    private static final String USECANCEL8_TAG = "useCancel8";
    private static final String USECANCEL5_TAG = "useCancel5";

    public static boolean parallel8 = true;
    public static boolean parallel5 = true;
    public static boolean reach8    = true;
    public static boolean reach5    = true;

    public static int     checkBeats = 2;    // 基本拍の2倍までチェック.
    public static boolean useCancel8 = true; // 途中で解消音程があれば連続8度としない.
    public static boolean useCancel5 = true; // 途中で解消音程があれば連続5度としない.

    public static int [] cancel8Intervals = {3,4,5,7,8,9}; // 連続8度の解消音程.
    public static int [] cancel5Intervals = {3,4,5,8,9};   // 連続5度の解消音程.

    /////////////////////////////////////////////////////////////////////

    public  static void load()
    {
        File file = new File(Kit.HOME + FILE_NAME);

        if (!file.exists())
        {
            save();
            return;
        }

        Properties prop = new Properties();

        try (InputStreamReader in
                = new InputStreamReader(new FileInputStream(file), "UTF-8"))
        {
            prop.load(in);

            String wk = prop.getProperty(PARALLELE8_TAG);
            if (wk != null)
                parallel8 = Boolean.parseBoolean(wk);

            wk = prop.getProperty(PARALLELE5_TAG);
            if (wk != null)
                parallel8 = Boolean.parseBoolean(wk);

            wk = prop.getProperty(REACH8_TAG);
            if (wk != null)
                reach8 = Boolean.parseBoolean(wk);

            wk = prop.getProperty(REACH5_TAG);
            if (wk != null)
                reach5 = Boolean.parseBoolean(wk);

            wk = prop.getProperty(CHECKBEATS_TAG);
            if (wk != null)
                checkBeats = Integer.parseInt(wk);

            wk = prop.getProperty(USECANCEL8_TAG);
            if (wk != null)
                useCancel8 = Boolean.parseBoolean(wk);

            wk = prop.getProperty(USECANCEL5_TAG);
            if (wk != null)
                useCancel5 = Boolean.parseBoolean(wk);
        }
        catch(Exception ex)
        {
            Kit.print(ex.getMessage());
        }
    }

    public static void save()
    {
        File file = new File(Kit.HOME + FILE_NAME);
        try (BufferedWriter stream
             = new BufferedWriter(
                new OutputStreamWriter(
                 new FileOutputStream(file), "UTF-8")) )
        {
            stream.write(PARALLELE8_TAG  + ":" +  parallel8 + "\n");
            stream.write(PARALLELE5_TAG  + ":" +  parallel5 + "\n");
            stream.write(REACH8_TAG      + ":" +  reach8 + "\n");
            stream.write(REACH5_TAG      + ":" +  reach5 + "\n");

            stream.write(CHECKBEATS_TAG  + ":" +  checkBeats + "\n");
            stream.write(USECANCEL8_TAG  + ":" +  useCancel8 + "\n");
            stream.write(USECANCEL5_TAG  + ":" +  useCancel5 + "\n");
        }
        catch(Exception ex)
        {
            Kit.print(ex.getMessage());
        }
    }

    ////////////////////////////////////////////////////////////

    public static int getCheckDuration(NoteCreator creator)
    {
        if (creator.beatType == 2)
        {
            if (creator.beats == 6
             || creator.beats == 9)
                return checkBeats * 48 * 3;
            else // 4/2 3/2 2/2
                return checkBeats * 48;
        }
        if (creator.beatType == 4)
        {
            if (creator.beats == 6
             || creator.beats == 9)
                return checkBeats * 24 * 3;
            else // 4/4 3/4 2/4
                return checkBeats * 24;
        }
        if (creator.beatType == 8)
        {
            if (creator.beats == 6
             || creator.beats == 9)
                // 6/8  9/8
                return checkBeats * 12 * 3;
            else // 4/8 3/8 2/8
                return checkBeats * 12;
        }
        else
        {
            return checkBeats * 24;
        }
    }
}
