//============================================================================
//
//  Parallel checker
//
//  Checker.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import  java.io.*;
import  java.util.*;
import static parallel.Constant.*;

public class Checker
{
    MuseXMLLoader loader = new MuseXMLLoader();
    NoteCreator   tracks = new NoteCreator();

    public Checker()
    {
    }

    public void loadXML(File musicXmlFile)
    {
        try
        {
            loader.parse(musicXmlFile.getAbsolutePath());
            Kit.print("読込み OK.");
        }
        catch(Exception ex)
        {
            System.err.println(ex);
            ex.printStackTrace();
            Kit.print("読込み Error:" + ex.getMessage());
        }
        catch(Error err)
        {
            System.err.println(err);
            err.printStackTrace();
            Kit.print("読込み Error:" + err.getMessage());
        }
    }

    public void run()
    {
        try
        {
            Kit.setWaitCursor(true);
            if (loader.parts == null || loader.parts.size() == 0)
            {
                Kit.print("MusicXMLが読み込まれていません.");
                return;
            }
            Kit.print(">>>>");

            tracks.prepare();

            tracks.create(loader.parts);

            check();

            Kit.print("<<<<");
        }
        catch(Exception ex)
        {
            System.err.println(ex);
            ex.printStackTrace();
        }
        catch(Error err)
        {
            System.err.println(err);
            err.printStackTrace();
        }
        finally
        {
            Kit.setWaitCursor(false);
        }
    }

    private void check()
    {
        ArrayList<ArrayList<OnPointPair>> result = tracks.result;

        if (Options.parallel8)
        {
            Parallel8 checker = new Parallel8();
            checker.check(tracks);
        }
        if (Options.parallel5)
        {
            Parallel5 checker = new Parallel5();
            checker.check(tracks);
        }
        Kit.putErr();
        Kit.clearErr();

        if (Options.reach8)
        {
            Reach8 checker = new Reach8();
            checker.check(tracks);
        }
        if (Options.reach5)
        {
            Reach5 checker = new Reach5();
            checker.check(tracks);
        }
        Kit.putErr();
        Kit.clearErr();
    }
}
