//============================================================================
//
//  Parallel checker
//
//  Parallel8.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.io.*;
import java.util.*;
import static parallel.Constant.*;

// 連続8度を抽出する.
public class Parallel8
{
    int checkBeats;

    public Parallel8()
    {
    }

    public void check(NoteCreator creator)
    {
        checkBeats =  Options.getCheckDuration(creator);

        ArrayList<ArrayList<OnPointPair>> result = creator.result;
        for(int i = 0; i < result.size(); ++i)
        {
            ArrayList<OnPointPair> trackPair = result.get(i);
            checkPair(trackPair);
        }
    }

    private void checkPair(ArrayList<OnPointPair> onpts)
    {
        int s = onpts.size();
        for(int i = 0; i < s; ++i)
        {
            OnPointPair v1 = onpts.get(i);
            // 休符が含まれる場合.
            if (v1.hasRest())
                continue;

            int itv1 = v1.interval;
            // 完全1/8度の場合,
            // 以降のデータで完全1/8度をチェック.
            if (itv1 == 0 || itv1 == 12)
            {
                checkParallel8(onpts, i, v1);
            }
        }
    }

    void checkParallel8(ArrayList<OnPointPair> onpts, int st, OnPointPair v1)
    {
        int pos1 = v1.pos;
        int s = onpts.size();
        for(int i = 1; i < 12; ++i)
        {
            if (st + i >= s)
                break;
            OnPointPair v2 = onpts.get(st + i);
            int itv2 = v2.interval;
            int pos2 = v2.pos;

            // チェック範囲を超えた場合.
            if (Math.abs(pos2 - pos1) >= checkBeats)
                break;

            // 休符が含まれる場合.
            if (v2.hasRest())
                continue;

            // 連続以外.
            if (itv2 != 0 && itv2 != 12)
                continue;

            // 同音に回帰した場合.
            if (i > 1 && v1.isSameTone(v2))
                continue;

            // 同音程にOct跳躍した場合.
            if (v1.isDiagonalSame(v2))
                continue;

            // 前にOct跳躍が含まれる場合(チェック済み).
            if (i >= 2)
            {
                OnPointPair v3 = onpts.get(st + i -1);
                if (v2.hasOctave(v3))
                    continue;
            }

            if (!Options.useCancel5)
                Kit.addErr("連続８度", v1, v2);
            // 解消音程を使う場合.
            else if (!includeCancel(onpts, st, st + i))
                Kit.addErr("連続８度", v1, v2);
        }
    }

    boolean includeCancel(ArrayList<OnPointPair> onpts,
                          int si, int ei)
    {
        int [] intervals = Options.cancel8Intervals;
        for (int i = si + 1; i < ei;++i)
        {
            OnPointPair v = onpts.get(i);
            int itv = v.interval;
            for(int j = 0; j < intervals.length; ++j)
               if (itv == intervals[j])
                   return true;
        }
        return false;
    }

}
