//============================================================================
//
//  Parallel checker
//
//  Reach8
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.io.*;
import java.util.*;
import static parallel.Constant.*;

// 並達8度を抽出する.
public class Reach8
{
    public Reach8()
    {
    }

    public void check(NoteCreator creator)
    {
        ArrayList<ArrayList<OnPointPair>> result = creator.result;
        for(int i = 0; i < result.size(); ++i)
        {
            ArrayList<OnPointPair> trackPair = result.get(i);
            checkPair(trackPair);
        }
    }

    private void checkPair(ArrayList<OnPointPair> onpts)
    {
        int s = onpts.size();
        for(int i = 0; i < s; ++i)
        {
            OnPointPair v1 = onpts.get(i);
            // 休符が含まれる場合.
            if (v1.hasRest())
                continue;
            if (v1.note1.padding || v1.note2.padding)
                continue;

            if (i > 0)
            {
                int itv1 = v1.interval;
                if (itv1 == 0 || itv1 == 12)
                {
                    OnPointPair v2 = onpts.get(i - 1);
                    if (v2.hasRest())
                        continue;
                    if (v2.note1.padding || v2.note2.padding)
                        continue;
                    checkReach8(v2, v1);
                }
            }
        }
    }

    void checkReach8(OnPointPair v1, OnPointPair v2)
    {
        int diff1 = v1.note1.no - v2.note1.no;
        int diff2 = v1.note2.no - v2.note2.no;

        // Oct跳躍あり.
        if (diff1 % 12 == 0 || diff2 % 12 == 0)
            return;

        if (diff1 < 0 && diff2 >= 0)
            return;
        if (diff1 >= 0 && diff2 < 0)
            return;

        diff1 = Math.abs(diff1);
        diff2 = Math.abs(diff2);
        if (diff1 <= 2 || diff2 <= 2)
            return;

        Kit.addErr("並達８度", v1, v2);
    }

}
