//============================================================================
//
//  Parallel checker
//
//  MuseXMLLoader.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;

public class MuseXMLLoader
{
    ArrayList<Object[]> parts;

    public void parse(String musicXmlFile) throws Exception
    {
        Kit.print("ファイル : " + musicXmlFile.replace("\\", "/"));
        File file = new File(musicXmlFile);
        if (!file.exists())
        {
            String errMsg = String.format("xml can't load! : path = %s", musicXmlFile);
            throw new Exception(errMsg);
        }

        parts = new  ArrayList<Object[]>();

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(false);
        // dtdファイルの読み込みをSkip.
        factory.setValidating(false);
        factory.setFeature("http://xml.org/sax/features/namespaces", false);
        factory.setFeature("http://xml.org/sax/features/validation", false);
        factory.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
        factory.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);

        DocumentBuilder b = factory.newDocumentBuilder();
        Document xmldoc = b.parse(file);
        NodeList nl0  = xmldoc.getChildNodes();
        for (int i = 0; i < nl0.getLength(); ++i)
        {
            org.w3c.dom.Node n0 = nl0.item(i);
            short t0 = n0.getNodeType();
            if (t0 != org.w3c.dom.Node.ELEMENT_NODE)
                continue;

            if (n0.getNodeName().equals("score-partwise"))
            {
                NodeList n1  = n0.getChildNodes();
                for (int j = 0; j < n1.getLength(); ++j)
                {
                    org.w3c.dom.Node n2 = n1.item(j);
                    short t2 = n2.getNodeType();
                    if (t2 != org.w3c.dom.Node.ELEMENT_NODE)
                        continue;

                    String nodeName = n2.getNodeName();
                    if (n2.getNodeName().equals("part-list"))
                    {
                        getPartList(n2, parts);
                    }
                    else if (n2.getNodeName().equals("part"))
                    {
                        ArrayList<Measure> m = collectNotes(n2);

                        NamedNodeMap att = n2.getAttributes();
                        String name = att.getNamedItem("id").getNodeValue();

                        for(int k = 0;k < parts.size(); ++k)
                        {
                            Object[] pp = parts.get(k);
                            if (pp[0].equals(name))
                            {
                                pp[1] = m;
                                break;
                            }
                        }
                    }
                }
            }
        }
    }

    private void getPartList(org.w3c.dom.Node partList,
                             ArrayList<Object[]> parts) throws Exception
    {
        NodeList nl  = partList.getChildNodes();
        for (int i = 0; i < nl.getLength(); ++i)
        {
            org.w3c.dom.Node n = nl.item(i);
            short t = n.getNodeType();
            if (t != org.w3c.dom.Node.ELEMENT_NODE)
                continue;

            String nodeName = n.getNodeName();
            NamedNodeMap att = n.getAttributes();

            if (nodeName.equals("score-part"))
            {
                org.w3c.dom.Node id = att.getNamedItem("id");
                parts.add(new Object[] {id.getNodeValue(), null});
            }
        }
    }

    int curr_beats;
    int curr_beatType;
    int curr_divisions;

    private ArrayList<Measure> collectNotes(org.w3c.dom.Node parts) throws Exception
    {
        ArrayList<Measure> mes = new ArrayList<Measure>();
        Measure bar = null;
        NodeList nl  =  parts.getChildNodes();
        for (int i = 0; i < nl.getLength(); ++i)
        {
            org.w3c.dom.Node n = nl.item(i);
            short t = n.getNodeType();
            if (t != org.w3c.dom.Node.ELEMENT_NODE)
                continue;

            String nodeName = n.getNodeName();
            NamedNodeMap att = n.getAttributes();

            if (nodeName.equals("measure"))
            {
                if (bar != null)
                    mes.add(bar);
                bar = new Measure();
                org.w3c.dom.Node no = att.getNamedItem("number");
                bar.no = Integer.parseInt(no.getNodeValue());

                NodeList nl2  = n.getChildNodes();
                for (int j = 0; j < nl2.getLength(); ++j)
                {
                    org.w3c.dom.Node n2 = nl2.item(j);
                    short t2 = n2.getNodeType();
                    if (t2 != org.w3c.dom.Node.ELEMENT_NODE)
                        continue;

                    String nodeName2 = n2.getNodeName();
                    if (nodeName2.equals("attributes"))
                    {
                        getBeat(n2, bar);
                    }
                    else if (nodeName2.equals("note"))
                    {
                        getNote(n2, bar);
                    }
                    else if (nodeName2.equals("backup"))
                    {
                        getBackup(n2, bar);
                    }
                }
            }
            bar.divisions = curr_divisions;
            bar.beats     = curr_beats;
            bar.beatType  = curr_beatType;
        }
        if (bar != null)
            mes.add(bar);
        return mes;
    }

    private void getBeat(org.w3c.dom.Node measure,Measure bar) throws Exception
    {
        NodeList nl0  = measure.getChildNodes();
        for (int i = 0; i < nl0.getLength(); ++i)
        {
            org.w3c.dom.Node n0 = nl0.item(i);
            short t0 = n0.getNodeType();
            if (t0 != org.w3c.dom.Node.ELEMENT_NODE)
                continue;

            String nodeName = n0.getNodeName();
            if (nodeName.equals("divisions"))
            {
                Node textNode = n0.getFirstChild();
                bar.divisions = Integer.parseInt(textNode.getNodeValue());
                curr_divisions = bar.divisions;
            }
            else if (nodeName.equals("time"))
            {
                NodeList nl1  = n0.getChildNodes();
                for (int j = 0; j < nl1.getLength(); ++j)
                {
                    org.w3c.dom.Node n1 = nl1.item(j);
                    short t1 = n1.getNodeType();
                    if (t1 != org.w3c.dom.Node.ELEMENT_NODE)
                        continue;

                    String nodeName1 = n1.getNodeName();
                    if (nodeName1.equals("beats"))
                    {
                        Node textNode = n1.getFirstChild();
                        bar.beats = Integer.parseInt(textNode.getNodeValue());
                        curr_beats = bar.beats;
                    }
                    else if (nodeName1.equals("beat-type"))
                    {
                        Node textNode = n1.getFirstChild();
                        bar.beatType = Integer.parseInt(textNode.getNodeValue());
                        curr_beatType = bar.beatType;
                    }
                }
            }
        }
    }

    private void getNote(org.w3c.dom.Node note,Measure bar) throws Exception
    {
        XMLNote n = new XMLNote();
        boolean isGrace = false;

        NodeList nl0  = note.getChildNodes();
        for (int i = 0; i < nl0.getLength(); ++i)
        {
            org.w3c.dom.Node n0 = nl0.item(i);
            short t0 = n0.getNodeType();
            if (t0 != org.w3c.dom.Node.ELEMENT_NODE)
                continue;

            String nodeName = n0.getNodeName();
            if (nodeName.equals("duration"))
            {
                Node textNode = n0.getFirstChild();
                n.dur = textNode.getNodeValue();
            }
            else if (nodeName.equals("voice"))
            {
                Node textNode = n0.getFirstChild();
                n.voice = textNode.getNodeValue();
            }
            else if (nodeName.equals("chord"))
            {
                // 和弦の場合.
                n.isChord = true;
            }
            else if (nodeName.equals("grace"))
            {
                // 装飾音は無視.
                isGrace = true;
            }
            else if (nodeName.equals("pitch"))
            {
                NodeList nl1  = n0.getChildNodes();
                for (int j = 0; j < nl1.getLength(); ++j)
                {
                    org.w3c.dom.Node n1 = nl1.item(j);
                    short t1 = n1.getNodeType();
                    if (t1 != org.w3c.dom.Node.ELEMENT_NODE)
                        continue;

                    String nodeName1 = n1.getNodeName();
                    if (nodeName1.equals("step"))
                    {
                        Node textNode = n1.getFirstChild();
                        n.step = textNode.getNodeValue();
                    }
                    if (nodeName1.equals("octave"))
                    {
                        Node textNode = n1.getFirstChild();
                        n.octave = textNode.getNodeValue();
                    }
                    else if (nodeName1.equals("alter"))
                    {
                        Node textNode = n1.getFirstChild();
                        n.alter = textNode.getNodeValue();
                    }
                }
            }
        }
        if (!isGrace)
        {
            bar.voiceSet.add(n.voice);
            bar.notes.add(n);
        }
    }

    private void getBackup(org.w3c.dom.Node backup, Measure bar) throws Exception
    {
        XMLNote n = new XMLNote();
        n.isBackup = true;

        NodeList nl0  = backup.getChildNodes();
        for (int i = 0; i < nl0.getLength(); ++i)
        {
            org.w3c.dom.Node n0 = nl0.item(i);
            short t0 = n0.getNodeType();
            if (t0 != org.w3c.dom.Node.ELEMENT_NODE)
                continue;

            String nodeName = n0.getNodeName();
            if (nodeName.equals("duration"))
            {
                Node textNode = n0.getFirstChild();
                n.dur = textNode.getNodeValue();
            }
        }

        bar.notes.add(n);
    }

}
