//============================================================================
//
//  Parallel checker
//
//  OnPointPair.java
//
//  Copyright (c) 2019 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package parallel;

import java.util.*;

// 2Track間で,発音位置のペア.
public class OnPointPair
{
    public int  pos;      // 先頭からのTick数.
    public int  interval; // 音程.
    public Note note1;    // ペア先頭Trackの音符.
    public Note note2;    // ペア次のTrackの音符.

    public OnPointPair(int pos, int interval, Note note1, Note note2)
    {
        this.pos   = pos;
        this.interval = interval;
        this.note1 = note1;
        this.note2 = note2;
    }

    public String toString()
    {
        int bpos = (pos - note1.bar.totalTick);
        int unit = 24;
        if (note1.bar.beatType == 8)
            unit = 12;
        else if (note1.bar.beatType == 2)
            unit = 48;

        return String.format(" %2d小節,%2d拍目,%2d-Tick : Track%d (%s) <---> Track%d (%s)",
                                     note1.bar.no,
                                     bpos / unit + 1,
                                     bpos % unit,
                                     note1.track+1, note1.sig,
                                     note2.track+1, note2.sig);
    }

    public boolean hasRest()
    {
        if (this.note1.no == -1 || this.note2.no == -1)
            return true;
        return false;
    }

    public boolean isSameTone(OnPointPair rhs)
    {
        return this.note1.no == rhs.note1.no
            && this.note2.no == rhs.note2.no;
    }

    public boolean isDiagonalSame(OnPointPair rhs)
    {
        if (this.note1.no == rhs.note1.no)
            return ((this.note2.no - rhs.note2.no) % 12 == 0);
        else if (this.note2.no == rhs.note2.no)
            return ((this.note1.no - rhs.note1.no) % 12 == 0);
        else
            return false;
    }

    public boolean hasOctave(OnPointPair rhs)
    {
        if (Math.abs(this.note2.no - rhs.note2.no) == 12)
            return true;
        if (Math.abs(this.note1.no - rhs.note1.no) == 12)
            return true;
        return false;
    }
}

